#!/usr/bin/env python

"""
synopsis:
    Client that requests document services.
    Use the Python cmd module and a command shell for a user interface.
    Use doc_server_lib.py for database access.
    Proved the following commands that the user can enter at the prompt:
    - add document.
    - list all documents in the database.
    - get document by ID.
    - search for documents with a given tag.
usage:
    python doc_cmd_client01.py <databasefile>
options:
    None
examples:
    python doc_cmd_client01.py my_database.sqlite
"""

import sys
import getopt
from cmd import Cmd
import doc_database_lib


class DocServeCmdLine(Cmd):
    def __init__(self, db_name):
        Cmd.__init__(self)
        self.prompt = 'Db cmd: '
        self.database = doc_database_lib.DocDatabase()
        self.database.connect(db_name)

    do_h = Cmd.do_help

    def do_list(self, args):
        """Retrieve and show a list of all documents.  Show ids, tags,
            and descriptions.
        usage: list
        """
        docs = self.database.list()
        for doc in docs:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
        return False

    def do_get(self, args):
        """Retrieve a single document by ID.  Show the document ID, tags,
            description, and body.
        usage: get <id>
        """
        args = args.split()
        if len(args) != 1:
            return False
        id = int(args[0])
        doc = self.database.get(id)
        if doc is None:
            print 'document {} not found'.format(id)
        else:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
            print '    body:        {}'.format(doc[3])
        return False

    def do_search(self, args):
        """Search for a set of documents by tag.
        usage: search <tag>
        """
        args = args.split()
        if len(args) != 1:
            return False
        tag = args[0]
        docs = self.database.search(tag)
        for doc in docs:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
        return False

    def do_add(self, args):
        """Add a new document to the repository.
        usage: add <tags>::<description>::<filename>
        """
        args = args.split('::')
        if len(args) != 3:
            return False
        tags = args[0]
        description = args[1]
        filename = args[2]
        body = None
        with open(filename, 'r') as infile:
            body = infile.read()
        reply = self.database.add(tags, description, body)
        print 'reply: {}'.format(reply)
        return False

##     def emptyline(self):
##         pass

    def do_quit(self, args):
        """Say goodbye, close the database, and quit.
        """
        print 'bye'
        self.database.close()
        # Return True in order to exit from the command loop.
        return True
    do_exit = do_quit


def start(db_name):
    """Start up the command line client.
    """
    cmdline = DocServeCmdLine(db_name)
    cmdline.cmdloop('Welcome to the doc server.')


def usage():
    sys.exit(__doc__)


def main():
    args = sys.argv[1:]
    try:
        opts, args = getopt.getopt(args, 'h', ['help', ])
    except:
        usage()
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()
    if len(args) != 1:
        usage()
    db_name = args[0]
    start(db_name)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
