#!/usr/bin/env python

"""
synopsis:
    Client that requests document services.
    Use ZeroMQ as a transport layer.
    Request the following services:
    - add document.
    - list all documents in the database.
    - get document by ID.
    - search for documents with a given tag.
usage:
    python zmq_doc_client01.py [options] command arg1 arg2 ...
options:
    None
examples:
    python zmq_doc_client01.py add "tag tag" "description" <filename>
    python zmq_doc_client01.py list
    python zmq_doc_client01.py get <id>
    python zmq_doc_client01.py search <tag>
"""

import sys
import getopt
from cmd import Cmd
import zmq_doc_client01 as doc_client


class DocServeCmdLine(Cmd):
    def __init__(self):
        Cmd.__init__(self)
        self.prompt = 'Enter cmd: '

    do_h = Cmd.do_help

    def do_list(self, args):
        """Retrieve and show a list of all documents.  Show ids, tags,
            and descriptions.
        usage: list
        """
        docs = doc_client.list_request([], show=False)
        for doc in docs:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
        return False

    def do_get(self, args):
        """Retrieve a single document by ID.  Show the document ID, tags,
            description, and body.
        usage: get <id>
        """
        args = args.split()
        if len(args) != 1:
            return False
        id = int(args[0])
        doc = doc_client.get_request([id], show=False)
        if doc is None:
            print 'document {} not found'.format(id)
        else:
            print 'document:'
            print '    id:          {}'.format(doc['id'])
            print '    tags:        {}'.format(doc['tags'])
            print '    description: {}'.format(doc['description'])
            print '    body:        {}'.format(doc['body'])
        return False

    def do_search(self, args):
        """Search for a set of documents by tag.
        usage: search <tag>
        """
        args = args.split()
        if len(args) != 1:
            return False
        tag = args[0]
        docs = doc_client.search_request([tag], show=False)
        for doc in docs:
            print 'document:'
            print '    id:          {}'.format(doc[0])
            print '    tags:        {}'.format(doc[1])
            print '    description: {}'.format(doc[2])
        return False

    def do_add(self, args):
        """Add a new document to the repository.
        usage: add <tags>::<description>::<filename>
        """
        args = args.split('::')
        if len(args) != 3:
            return False
        tags = args[0]
        description = args[1]
        filename = args[2]
        message = None
        args = [tags, description, filename]
        try:
            print '(cmd) sending request: "{}"'.format(args)
            message = doc_client.add_request(args, show=False)
        except IOError:
            print "can't read file: {}".format(filename)
        if message is not None:
            print 'message: {}'.format(message)
        return False

    def emptyline(self):
        pass

    def do_quit(self, args):
        """Say goodbye and quit.
        """
        print 'bye'
        # Return True in order to exit from the command loop.
        return True
    do_exit = do_quit


def start():
    """Start up the command line client.
    """
    cmdline = DocServeCmdLine()
    cmdline.cmdloop('Welcome to the doc server.')


def usage():
    sys.exit(__doc__)


def main():
    args = sys.argv[1:]
    try:
        opts, args = getopt.getopt(args, 'h', ['help', ])
    except:
        usage()
    for opt, val in opts:
        if opt in ('-h', '--help'):
            usage()
    if len(args) != 0:
        usage()
    start()


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
