#!/usr/bin/env python

"""
synopsis:
    Utility functions for h5py.
examples:
    ./h5py_utilities.py -s mydata.hdf5.xml
    ./h5py_utilities.py -o mydata.hdf5.xml path1 path2 ...
    ./h5py_utilities.py -c -f --outfile=tmp1.csv test1.hdf5 /grp1/grp2/dataset1
"""

import sys
import os
import csv
import argparse
import h5py


#
# Globals

Horizontal_rule = '-' * 60


def generate_csv(args):
    infilename = args.infilename
    infile = h5py.File(infilename, 'r')
    path = args.paths[0]
    obj = get_hdf5_object(infile, path)
    write_csv(obj, args)


def write_csv(obj, args):
    writer = csv.writer(args.outfile)
    if len(obj.shape) == 1:
        writer.writerow(obj)
    elif len(obj.shape) == 2:
        for row in obj:
            writer.writerow(row)
    else:
        sys.stderr.write("error -- can't handle objects with dim > 2\n")
    if args.outfile is not sys.stdout:
        args.outfile.close()


def show_object(infilename, paths, display_fn):
    infile = h5py.File(infilename, 'r')
    for path in paths:
        obj = get_hdf5_object(infile, path)
        display_fn(obj)


def get_hdf5_object(root_group, path):
    if not path.startswith('/'):
        sys.stderr.write('path "{}" does not start with "/"\n'.format(path))
        return None
    pathlist = path.split('/')[1:]
    objectname = pathlist.pop()
    group = root_group
    for pathspec in pathlist:
        group = group[pathspec]
    object = group[objectname]
    return object


def display_object(obj):
    if isinstance(obj, h5py.Dataset):
        print 'dataset name: {0}  dtype: {1}  shape: {2}'.format(
            obj.name, obj.dtype, obj.shape)
    elif isinstance(obj, h5py.Group):
        print 'group name: {0}'.format(
            obj.name, )
    else:
        print 'other'


def show_all_paths(infilename):
    infile = h5py.File(infilename, 'r')
    items = []
    collector = make_collector(items)
    infile.visititems(collector)
    for name, obj in items:
        if isinstance(obj, h5py.Group):
            print 'group:   /{}'.format(name)
        elif isinstance(obj, h5py.Dataset):
            print 'dataset: /{}'.format(name)
        else:
            # ? never happens?
            print 'other: {}'.format(name)


def make_collector(collection):
    def collect_fn(name, obj):
        collection.append((name, obj))
    return collect_fn


def show_divider(msg):
    print Horizontal_rule
    print msg.center(60)
    print Horizontal_rule


def open_file(args):
    if not args.force:
        if os.path.exists(args.outfile):
            sys.exit('\noutput file exists.  use -f/--force to overwrite.\n')
    outfile = open(args.outfile, 'wb')
    return outfile


def main():
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )
    parser.add_argument(
        'infilename', type=str,
        help='hdf5 input file name')
    parser.add_argument(
        'paths', type=str, nargs='*', metavar='path',
        help='paths (names) of objects to be shown (option -s)')
    parser.add_argument(
        '-s', '--show-all-paths', dest='show_all_paths',
        action='store_true', default=False,
        help='show the names/paths of all objects (groups and datasets)')
    parser.add_argument(
        '-c', '--generate_csv',   # dest='show_all_paths',
        action='store_true', default=False,
        help='show the names/paths of all objects (groups and datasets)')
    parser.add_argument(
        '-o', '--show-objects', dest='show_objects',
        action='store_true', default=False,
        help='show information on each object specified by paths')
    parser.add_argument(
        '--outfile', type=str, default=None,
        help='output file (default: sys.stdout)')
    parser.add_argument(
        '-f', '--force', default=False, action='store_true',
        help='force write to output file')
    args = parser.parse_args()
    if args.outfile is None or args.outfile == '-':
        args.outfile = sys.stdout
    else:
        args.outfile = open_file(args)
    if args.show_all_paths:
        show_divider('all paths')
        show_all_paths(args.infilename)
    if args.show_objects:
        show_divider('specific objects')
        show_object(args.infilename, args.paths, display_object)
    if args.generate_csv:
        generate_csv(args)


if __name__ == '__main__':
    # dbg
    #import pdb; pdb.set_trace()
    #import ipdb; ipdb.set_trace()
    main()
