#!/usr/bin/env python

"""
usage: hdf5_to_json.py [-h] [-o OUTFILE] [-f] [-p] [-v] infile

Read an hdf5 file and write out json.  The json is a recusively
nested dictionary where:
    - keys are the names of groups and datasets and
    - values are dictionaries (for subgroups) and lists or lists
      of lists (for datasets).

positional arguments:
  infile                xml input file name

optional arguments:
  -h, --help            show this help message and exit
  -o OUTFILE, --outfile OUTFILE
                        output file name (default is sys.stdout)
  -f, --force           force overwrite of output file
  -p, --complete-path   use complete path as json dictionary keys
  -v, --verbose         print out dataset names

Example:
    python hdf5_to_json.py mydata.hdf5
    python hdf5_to_json.py -v -f -o outdata.json mydata.hdf5
"""

import sys
import os
import argparse
import h5py
import json
import numpy as np


Integer_types = set([
    int,
    np.int,
    np.int8,
    np.int16,
    np.int32,
    np.int64,
    ])
Float_types = set([
    float,
    np.float,
    np.float16,
    np.float32,
    np.float64,
    np.float128,
    ])
Str_types = set([
    str,
    np.str,
    ])


def get_datatype(dataset):
    dtype = dataset.dtype.type
    if dtype in Integer_types:
        datatype = int
    elif dtype in Float_types:
        datatype = float
    elif dtype in Str_types:
        datatype = str
    else:
        datatype = None
    return datatype
    #
    # must add other types.


def reduce_name(name):
    name1 = name.split('/')[-1]
    return name1 if name1 else '/'


def do_dataset(container, dataset, args):
    name = dataset.name
    if args.verbose:
        print '{}dataset: {}'.format(args.indent, dataset.name)
    shape = dataset.shape
    datatype = get_datatype(dataset)
    if datatype is None:
        sys.stderr.write('unknown dtype in dataset "{}"\n'.format(name))
    data = dataset.value.tolist()
    if len(shape) == 1:
        items = [datatype(x) for x in data]
    elif len(shape) == 2:
        items = [[datatype(y) for y in x] for x in data]
    if items:
        if args.complete_path:
            container[name] = items
        else:
            name1 = reduce_name(name)
            container[name1] = items


def sort_items_by_type(group):
    groups = []
    datasets = []
    for item in group.itervalues():
        if isinstance(item, h5py.Dataset):
            datasets.append(item)
        elif isinstance(item, h5py.Group):
            groups.append(item)
    return groups, datasets


def do_group(container, group, args):
    if args.verbose:
        print '{}group: {}'.format(args.indent, group.name)
    groups, datasets = sort_items_by_type(group)
    args.indent += '    '
    for dataset in datasets:
        do_dataset(container, dataset, args)
    for sub_group in groups:
        name = sub_group.name
        container1 = {}
        if args.complete_path:
            container[name] = container1
        else:
            name1 = reduce_name(name)
            container[name1] = container1
        do_group(container1, sub_group, args)


def hdf5_to_json(args):
    args.indent = ''
    infilename = args.infile
    infile = h5py.File(infilename, 'r')
    root_group = infile
    container = {'/': {}}
    container1 = container['/']
    do_group(container1, root_group, args)
    json.dump(container, args.outfile)


def make_file(args):
    if args.outfile is None:
        outfile = sys.stdout
    else:
        if not args.force:
            if os.path.exists(args.outfile):
                sys.exit(
                    'output file {} exists; to overwrite, use -f/--force'.
                    format(args.outfile))
        outfile = open(args.outfile, 'w')
    args.outfile = outfile


Synopsis = '''\
Read an hdf5 file and write out json.  The json is a recusively
nested dictionary where:
    - keys are the names of groups and datasets and
    - values are dictionaries (for subgroups) and lists or lists
      of lists (for datasets).
'''
Epilog = '''\
Example:
    python hdf5_to_json.py mydata.hdf5
    python hdf5_to_json.py -v -f -o outdata.json mydata.hdf5
'''


def main():
    parser = argparse.ArgumentParser(
        description=Synopsis,
        epilog=Epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )
    parser.add_argument(
        'infile', type=str,
        help='xml input file name')
    parser.add_argument(
        '-o', '--outfile', type=str,
        required=False, default=None,
        help='output file name (default is sys.stdout)')
    parser.add_argument(
        '-f', '--force', dest='force',
        action='store_true', default=False,
        help='force overwrite of output file')
    parser.add_argument(
        '-p', '--complete-path', dest='complete_path',
        action='store_true', default=False,
        help='use complete path as json dictionary keys')
    parser.add_argument(
        '-v', '--verbose', dest='verbose',
        action='store_true', default=False,
        help='print out dataset names')
    args = parser.parse_args()
    make_file(args)
    hdf5_to_json(args)
    args.outfile.close()


if __name__ == '__main__':
    # dbg
##     import ipdb
##     ipdb.set_trace()
    main()
