#!/usr/bin/env python

"""
usage: hdf5pylib_utilities.py [-h] [-s] [-o] infilename [paths [paths ...]]

Utility functions for h5py.

positional arguments:
  infilename            hdf5 input file name
  paths                 paths (names) of objects to be shown (option -s)

optional arguments:
  -h, --help            show this help message and exit
  -s, --show-all-paths  show the names/paths of all objects (groups and
                        datasets)
  -o, --show-objects    show information on each object specified by paths

Example:
    python hdf5pylib_utilities.py -s mydata.hdf5.xml
    python hdf5pylib_utilities.py -o mydata.hdf5.xml path1 path2 ...
"""

import sys
import argparse
import hdf5pylib


#
# Globals

Horizontal_rule = '-' * 60


def show_object(infilename, paths, display_fn):
    root = hdf5pylib.parse(infilename, silence=True)
    root_group = root.get_RootGroup()
    for path in paths:
        obj = get_hdf5_object(root_group, path)
        if obj is None:
            print 'Cannot find object: {}'.format(path)
        else:
            display_fn(path, obj)


def get_hdf5_object(root_group, path):
    if not path.startswith('/'):
        sys.stderr.write('path "{}" does not start with "/"\n'.format(path))
        return None
    pathlist = path.split('/')[1:]
    obj = get_hdf5_object_1(root_group, pathlist)
    return obj


def get_hdf5_object_1(obj, pathlist):
    if not pathlist or isinstance(obj, hdf5pylib.Dataset):
        return obj
    if len(pathlist) == 1:
        name = pathlist[0]
        for dataset in obj.get_Dataset():
            if dataset.get_Name() == name:
                return dataset
        for sub_group in obj.get_Group():
            if sub_group.get_Name() == name:
                return sub_group
    else:
        name = pathlist[0]
        for sub_group in obj.get_Group():
            if sub_group.get_Name() == name:
                return get_hdf5_object_1(sub_group, pathlist[1:])
    return None


def display_object(path, obj):
    if isinstance(obj, hdf5pylib.Dataset):
        print 'dataset name: {}  path: {}  dtype: {}'.format(
            obj.get_Name(), path, obj.get_DataType(), )
    elif isinstance(obj, hdf5pylib.Group):
        print 'group name: {}  path: {}'.format(
            obj.name, path, )
    else:
        print 'other'


def show_all_paths(infilename):
    root = hdf5pylib.parse(infilename, silence=True)
    root_group = root.get_RootGroup()
    show_all_paths_1(root_group, '')


def show_all_paths_1(group, indent):
    for dataset in group.get_Dataset():
        print '{}dataset name: {}'.format(indent, dataset.get_Name())
    for sub_group in group.get_Group():
        print '{}group name:   {}'.format(indent, sub_group.get_Name())
        show_all_paths_1(sub_group, indent + '    ', )


def show_divider(msg):
    print Horizontal_rule
    print msg.center(60)
    print Horizontal_rule


Synopsis = '''\
Utility functions for h5py.
'''

Epilog = '''\
Example:
    python hdf5pylib_utilities.py -s mydata.hdf5.xml
    python hdf5pylib_utilities.py -o mydata.hdf5.xml path1 path2 ...
'''


def main():
    parser = argparse.ArgumentParser(
        description=Synopsis,
        epilog=Epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )
    parser.add_argument(
        'infilename', type=str,
        help='hdf5 input file name')
    parser.add_argument(
        'paths', type=str, nargs='*',
        help='paths (names) of objects to be shown (option -o)')
    parser.add_argument(
        '-s', '--show-all-paths', dest='show_all_paths',
        action='store_true', default=False,
        help='show the names/paths of all objects (groups and datasets)')
    parser.add_argument(
        '-o', '--show-objects', dest='show_objects',
        action='store_true', default=False,
        help='show information on each object specified by paths')
    args = parser.parse_args()
    if args.show_all_paths:
        show_divider('all paths')
        show_all_paths(args.infilename)
    if args.show_objects:
        show_divider('specific objects')
        show_object(args.infilename, args.paths, display_object)


if __name__ == '__main__':
    # dbg
    #import pdb
    #pdb.set_trace()
    main()
