#!/usr/bin/env python

"""
usage: xml_to_hdf5.py [-h] [-f] [-v] infilename outfilename

Read and parse HDF5 XML file (e.g. from `h5dump -x`) and
write to hdf5 file.

positional arguments:
  infilename     xml input file name (read from stdin if "-")
  outfilename    hdf5 output file name

optional arguments:
  -h, --help     show this help message and exit
  -f, --force    force overwrite of output file
  -v, --verbose  print out dataset names

Example:
    python xml_to_hdf5.py mydata.hdf5.xml mydata.hdf5
"""

import sys
import os
import re
import argparse
import hdf5pylib
import h5py
import numpy as np


def find_dataset(path, group):
    dataset = None
    if path:
        name = path.pop()
        if path:
            for sub_group in group.get_Group():
                if name == sub_group.get_Name():
                    dataset = find_dataset(path, sub_group)
        else:
            for dataset in group.get_Dataset():
                if name == dataset.get_Name():
                    dataset = dataset
    return dataset


def get_shape(dataset):
    dataspace = dataset.get_Dataspace()
    simpledataspace = dataspace.get_SimpleDataspace()
    shape = []
    for dimension in simpledataspace.get_Dimension():
        dimsize = int(dimension.get_DimSize())
        shape.append(dimsize)
    return shape


def get_dtype(dataset):
    datatype = dataset.get_DataType()
    atomictype = datatype.get_AtomicType()
    if atomictype.get_IntegerType():
        type_spec = atomictype.get_IntegerType()
        size = type_spec.get_Size()
        if size:
            size = int(size)
            if size == 1:
                dtype = np.int8
            elif size == 2:
                dtype = np.int16
            elif size == 4:
                dtype = np.int32
            elif size == 8:
                dtype = np.int64
    elif atomictype.get_FloatType():
        type_spec = atomictype.get_FloatType()
        size = type_spec.get_Size()
        if size:
            size = int(size)
            if size == 2:
                dtype = np.float16
            elif size == 4:
                dtype = np.float32
            elif size == 8:
                dtype = np.float64
            elif size == 16:
                dtype = np.float128
    elif atomictype.get_StringType():
        dtype = np.str
    else:
        dtype = None
    return dtype
    #
    # must add other types.


def do_dataset(container, dataset, args):
    name = dataset.get_Name()
    shape = get_shape(dataset)
    dtype = get_dtype(dataset)
    if dtype is None:
        sys.stderr.write('unknown dtype in dataset "{}"\n'.format(name))
    data = dataset.get_Data()
    datafromfile = data.get_DataFromFile()
    if datafromfile is None:
        items = None
    else:
        items = [
            dtype(item) for item in re.split('[\n ]+', datafromfile) if item
        ]
        if len(shape) == 2:
            rows, cols = shape
            count = rows * cols
            items = [items[n:n + cols] for n in xrange(0, count, cols)]
    if items:
        ds = container.create_dataset(name, shape, dtype)
        ds[...] = items
        if args.verbose:
            print 'created dataset: {}  shape: {}  dtype: {}'.format(
                ds.name, ds.shape, ds.dtype, )


def do_group(container, group, args):
    container1 = container.create_group(group.get_Name())
    for dataset in group.get_Dataset():
        do_dataset(container1, dataset, args)
    for sub_group in group.get_Group():
        do_group(container1, sub_group, args)


def do_root_group(container, root_group, args):
    for dataset in root_group.get_Dataset():
        # add the dataset
        do_dataset(container, dataset, args)
    for group in root_group.get_Group():
        do_group(container, group, args)


def make_file(args):
    outfilename = args.outfilename
    if not args.force:
        if os.path.exists(outfilename):
            sys.exit('output file exists; to overwrite, use -f/--force')
    outfile = h5py.File(outfilename, 'w')
    return outfile


def xml_to_hdf5(args):
    infilename = args.infilename
    if infilename == '-':
        infilename = sys.stdin
    outfile = make_file(args)
    root = hdf5pylib.parse(infilename, silence=True)
    root_group = root.get_RootGroup()
    if args.path_to_dataset:
        path = args.path_to_dataset.split('/')
        path.reverse()
        path.pop()
        dataset = find_dataset(path, root_group)
        if dataset is not None:
            pass
        else:
            sys.exit("can't find dataset: '{}'".format(args.path_to_dataset))
    else:
        do_root_group(outfile, root_group, args)
    outfile.close()


Synopsis = '''\
Read and parse HDF5 XML file (e.g. from `h5dump -x`) and
write to hdf5 file.
'''

Epilog = '''\
Example:
    python xml_to_hdf5.py mydata.hdf5.xml mydata.hdf5
'''


def main():
    parser = argparse.ArgumentParser(
        description=Synopsis,
        epilog=Epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        'infilename', type=str,
        help='xml input file name (read from stdin if "-")')
    parser.add_argument(
        'outfilename', type=str,
        help='hdf5 output file name')
    parser.add_argument(
        '-p', '--path-to-dataset', type=str, dest='path_to_dataset',
        required=False, default=None,
        help='path to a single dataset to be dumped to json')
    parser.add_argument(
        '-f', '--force', dest='force',
        action='store_true', default=False,
        help='force overwrite of output file')
    parser.add_argument(
        '-v', '--verbose', dest='verbose',
        action='store_true', default=False,
        help='print out dataset names')
    args = parser.parse_args()
    xml_to_hdf5(args)


if __name__ == '__main__':
    # dbg
##     import ipdb
##     ipdb.set_trace()
    main()
