#!/usr/bin/env python

"""
usage: xml_to_json.py [-h] [-o OUTFILE] [-f] [-v] infile

Read XML created by `h5dump -x` and write out json.  The json is a
recusively nested dictionary where:
    - keys are the names of groups and datasets and
    - values are dictionaries (for subgroups) and lists or lists
      of lists (for datasets).

positional arguments:
  infile                xml input file name (read from stdin if "-")

optional arguments:
  -h, --help            show this help message and exit
  -o OUTFILE, --outfile OUTFILE
                        output file name (default is sys.stdout)
  -f, --force           force overwrite of output file
  -v, --verbose         print out dataset names

Example:
    python xml_to_json.py mydata.hdf5.xml
    python xml_to_json.py -v -f -o outdata.json mydata.hdf5.xml
"""

import sys
import os
import re
import argparse
import hdf5pylib
import json


def find_dataset(path, group):
    dataset = None
    if path:
        name = path.pop()
        if path:
            for sub_group in group.get_Group():
                if name == sub_group.get_Name():
                    dataset = find_dataset(path, sub_group)
        else:
            for dataset in group.get_Dataset():
                if name == dataset.get_Name():
                    dataset = dataset
    return dataset


def get_shape(dataset):
    dataspace = dataset.get_Dataspace()
    simpledataspace = dataspace.get_SimpleDataspace()
    shape = []
    for dimension in simpledataspace.get_Dimension():
        dimsize = int(dimension.get_DimSize())
        shape.append(dimsize)
    return shape


def get_dtype(dataset):
    datatype = dataset.get_DataType()
    atomictype = datatype.get_AtomicType()
    if atomictype.get_IntegerType():
        dtype = int
    elif atomictype.get_FloatType():
        dtype = float
    elif atomictype.get_StringType():
        dtype = str
    else:
        dtype = None
    return dtype
    #
    # must add other types.


def do_dataset(container, dataset, args):
    name = dataset.get_Name()
    if args.verbose:
        print '{}dataset: {}'.format(args.indent, name)
    shape = get_shape(dataset)
    dtype = get_dtype(dataset)
    if dtype is None:
        sys.stderr.write('unknown dtype in dataset "{}"\n'.format(name))
    data = dataset.get_Data()
    datafromfile = data.get_DataFromFile()
    if datafromfile is None:
        items = None
    else:
        items = [
            dtype(item) for item in re.split('[\n ]+', datafromfile) if item
            ]
        if len(shape) == 2:
            rows, cols = shape
            count = rows * cols
            items = [items[n:n+cols] for n in xrange(0, count, cols)]
    if items:
        container[name] = items


def do_group(container, group, args):
    if args.verbose:
        print '{}group: {}'.format(args.indent, group.get_Name())
    args.indent += '    '
    for dataset in group.get_Dataset():
        do_dataset(container, dataset, args)
    for sub_group in group.get_Group():
        name = sub_group.get_Name()
        container1 = {}
        container[name] = container1
        do_group(container1, sub_group, args)


def xml_to_json(args):
    args.indent = ''
    infilename = args.infile
    if infilename == '-':
        infilename = sys.stdin
    root = hdf5pylib.parse(infilename, silence=True)
    root_group = root.get_RootGroup()
    if args.path_to_dataset:
        path = args.path_to_dataset.split('/')
        path.reverse()
        path.pop()
        dataset = find_dataset(path, root_group)
        if dataset is not None:
            container = {}
            do_dataset(container, dataset, args)
        else:
            sys.exit("can't find dataset: '{}'".format(args.path_to_dataset))
    else:
        container = {'/': {}}
        container1 = container['/']
        do_group(container1, root_group, args)
    json.dump(container, args.outfile)


def make_file(args):
    if args.outfile is None:
        outfile = sys.stdout
    else:
        if not args.force:
            if os.path.exists(args.outfile):
                sys.exit(
                    'output file {} exists; to overwrite, use -f/--force'.
                    format(args.outfile))
        outfile = open(args.outfile, 'w')
    args.outfile = outfile


Synopsis = '''\
Read XML created by `h5dump -x` and write out json.  The json is a
recusively nested dictionary where:
    - keys are the names of groups and datasets and
    - values are dictionaries (for subgroups) and lists or lists
      of lists (for datasets).
'''
Epilog = '''\
Example:
    python xml_to_json.py mydata.hdf5.xml
    python xml_to_json.py -v -f -o outdata.json mydata.hdf5.xml
'''


def main():
    parser = argparse.ArgumentParser(
        description=Synopsis,
        epilog=Epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )
    parser.add_argument(
        'infile', type=str,
        help='xml input file name (read from stdin if "-")')
    parser.add_argument(
        '-o', '--outfile', type=str,
        required=False, default=None,
        help='output file name (default is sys.stdout)')
    parser.add_argument(
        '-p', '--path-to-dataset', type=str, dest='path_to_dataset',
        required=False, default=None,
        help='path to a single dataset to be dumped to json')
    parser.add_argument(
        '-f', '--force', dest='force',
        action='store_true', default=False,
        help='force overwrite of output file')
    parser.add_argument(
        '-v', '--verbose', dest='verbose',
        action='store_true', default=False,
        help='print out dataset names')
    args = parser.parse_args()
    make_file(args)
    xml_to_json(args)
    args.outfile.close()


if __name__ == '__main__':
    # dbg
##     import ipdb
##     ipdb.set_trace()
    main()
