#!/usr/bin/env python

"""
synopsis:
    Worker process for the load-balancing broker.
    A worker process implemented in Python for the load-balancing broker.
    - Receives request message.
    - Counts elements in XML document, possibly filtering elements by tag.
    - Sends reply message with results.
"""

import sys
import re
import zmq
import json
from lxml import etree


BackAddress = 'tcp://127.0.0.1:12345'


def count_elements(root, tagfilter_pat):
    if (root.tag is not etree.Comment and
        (tagfilter_pat is None or
            tagfilter_pat.search(root.tag)is not None)):
        count = 1
    else:
        count = 0
    for node in root.iterdescendants():
        if (node.tag is not etree.Comment and
            (tagfilter_pat is None or
                tagfilter_pat.search(node.tag)is not None)):
            count += 1
    return count


def worker_task(ident):
    """Worker task, using a REQ socket to do load-balancing."""
    socket = zmq.Context().socket(zmq.REQ)
    socket.identity = u"Worker-{}".format(ident).encode("ascii")
    socket.connect(BackAddress)
    # Tell broker we're ready for work
    socket.send(b"READY")
    print 'Worker {} sent READY'.format(ident)
    while True:
        address, empty, request = socket.recv_multipart()
        payload = request.decode('ascii')
        payload = json.loads(payload)
        filename = payload['filename']
        tagfilter = payload.get('filter')
        if tagfilter is not None:
            tagfilter_pat = re.compile(tagfilter)
        else:
            tagfilter_pat = None
        try:
            doc = etree.parse(filename)
        except IOError:
            doc = None
        if doc is None:
            count = -1
        else:
            root = doc.getroot()
            count = count_elements(root, tagfilter_pat)
        payload = {
            'filename': filename,
            'filter': tagfilter,
            'count': count,
        }
        payload = json.dumps(payload)
        print 'worker {} sending payload: {}'.format(ident, payload)
        socket.send_multipart([address, b'', payload])


def main():
    args = sys.argv[1:]
    if len(args) != 1:
        sys.exit('usage: test03_worker.py <task_number>')
    ident = args[0]
    worker_task(ident)


if __name__ == "__main__":
    main()
