#!/usr/bin/env python

"""
synopsis:
    Compare synchronous and asynchronous code.
    Download a set of Web pages (by URL) and write the contents to files.
    Use the asynctools package (https://pypi.python.org/pypi/asynctools)
       to do this in a variety of ways: (1) using Parallel threads;
       (2) using a Pool of threads; (3) using a decorated function;
       and (4) using a serial, unoptimized algorithm.
usage:
    python async01_py2.py <command> [<arg> <args> ...]
    python async01_py3.py <command> [<arg> <args> ...]
commands:
    async
    async_pool <number_of_threads_in_pool>
    async_deco
    serial
examples:
    python async01_py2.py async
    python async01_py2.py async_pool 4
    python async01_py2.py async_deco
    python async01_py2.py serial
"""


from __future__ import print_function
import sys
import os
from asynctools.threading import Parallel, Async, Pool
import urllib


Links = [
    'http://www.python.org',
    'http://www.davekuhlman.org',
    'http://www.tornadoweb.org',
    'http://www.davekuhlman.org/python_multiprocessing_01.html',
    #'http://www.erlang.org/doc/',
    'http://www.ruby-lang.org',
    'https://developer.mozilla.org/en-US/docs/Web/JavaScript',
    'https://nodejs.org/en/',
]


def request(url, outfiledir, outfilestem, index):
    infile = urllib.urlopen(url)
    data = infile.read()
    infile.close()
    outfilename = '{}{:03d}.html'.format(outfilestem, index + 1)
    outfilename = os.path.join(outfiledir, outfilename)
    print('(request) url: {}  outfile: {}'.format(url, outfilename))
    with open(outfilename, 'w') as outfile:
        outfile.write(data)
    return data


@Async
def request_deco(url, outfiledir, outfilestem, index):
    infile = urllib.urlopen(url)
    data = infile.read()
    infile.close()
    outfilename = '{}{:03d}.html'.format(outfilestem, index + 1)
    outfilename = os.path.join(outfiledir, outfilename)
    print('(request) url: {}  outfile: {}'.format(url, outfilename))
    with open(outfilename, 'w') as outfile:
        outfile.write(data)
    return data


def test_async(outfiledir, outfilestem):
    pll = Parallel(request)
    for index, url in enumerate(Links):
        pll(url, outfiledir, outfilestem, index)  # Starts a new thread
    # Wait for the results
    results, errors = pll.join()


def test_async_pool(outfiledir, outfilestem, num_threads):
    pll = Pool(request, num_threads)
    for index, url in enumerate(Links):
        pll(url, outfiledir, outfilestem, index)  # Starts a new thread
    # Wait for the results
    results, errors = pll.join()


def test_async_deco(outfiledir, outfilestem):
    tasks = []
    for index, url in enumerate(Links):
        # Starts a new thread
        task = request_deco(url, outfiledir, outfilestem, index)
        tasks.append(task)
    # Wait for the results
    print('waiting for tasks')
    for task in tasks:
        task.wait()


def test_serial(outfiledir, outfilestem):
    for index, url in enumerate(Links):
        request(url, outfiledir, outfilestem, index)


def main():
    args = sys.argv[1:]
    if len(args) < 1:
        sys.exit(__doc__)
    command = args[0]
    outfiledir = 'DataSink'
    outfilestem = 'junk'
    if os.path.exists(outfiledir):
        if not os.path.isdir(outfiledir):
            sys.exit('\n"{}" exists and is not a directory.\n'.format(
                outfiledir))
    else:
        os.mkdir(outfiledir)
    if command == 'async':
        test_async(outfiledir, outfilestem)
    elif command == 'async_pool':
        if len(args) < 2:
            sys.exit(__doc__)
        num_threads = int(args[1])
        test_async_pool(outfiledir, outfilestem, num_threads)
    elif command == 'async_deco':
        test_async_deco(outfiledir, outfilestem)
    elif command == 'serial':
        test_serial(outfiledir, outfilestem)
    else:
        sys.exit(__doc__)


if __name__ == '__main__':
    main()
