#!/usr/bin/env python
"""
synopsis:
    Compare synchronous and asynchronous code.
    Effectively, compare blocking and non-blocking I/O.
    Download a set of Web pages (by URL) and write the contents to files.
    The asynchronous code uses asyncio and aiohttp
        (https://pypi.python.org/pypi/aiohttp).
usage:
    python asyncio01_py3.py <command>
commands:
    async
    serial
examples:
    python asyncio01_py3.py async
    python asyncio01_py3.py serial
    time python asyncio01_py3.py async
    time python asyncio01_py3.py serial
"""
import asyncio
import aiohttp
import sys
import os
import urllib.request


Links = [
    'http://www.python.org',
    'http://www.davekuhlman.org',
    'http://www.tornadoweb.org',
    'http://www.davekuhlman.org/python_multiprocessing_01.html',
    'http://www.ruby-lang.org',
    'https://developer.mozilla.org/en-US/docs/Web/JavaScript',
    'https://nodejs.org/en/',
]


@asyncio.coroutine
def request_async(url, outfiledir, outfilestem, index):
    response = yield from aiohttp.request('GET', url)
    data = yield from response.read()
    outfilename = '{}{:03d}.html'.format(outfilestem, index + 1)
    outfilename = os.path.join(outfiledir, outfilename)
    print('(request) url: {}  outfile: {}'.format(url, outfilename))
    with open(outfilename, 'w') as outfile:
        outfile.write(data.decode('utf-8'))
    return data


def request_serial(url, outfiledir, outfilestem, index):
    infile = urllib.request.urlopen(url)
    data = infile.read()
    infile.close()
    outfilename = '{}{:03d}.html'.format(outfilestem, index + 1)
    outfilename = os.path.join(outfiledir, outfilename)
    print('(request) url: {}  outfile: {}'.format(url, outfilename))
    with open(outfilename, 'w') as outfile:
        outfile.write(data.decode('utf-8'))
    return data


def test_async(outfiledir, outfilestem):
    print('(test_async) starting')
    # Instantiating tasks doesn't cause the coroutine to be run. It merely
    # schedules the tasks.
    tasks = []
    for index, url in enumerate(Links):
        tasks.append(
            asyncio.Task(request_async(url, outfiledir, outfilestem, index)))
    # Get the event loop and cause it to run until all the tasks are done.
    loop = asyncio.get_event_loop()
    loop.run_until_complete(asyncio.wait(tasks))
    loop.close()
    print('(test_async) finished')


def test_serial(outfiledir, outfilestem):
    print('(test_serial) starting')
    for index, url in enumerate(Links):
        request_serial(url, outfiledir, outfilestem, index)
    print('(test_serial) finished')


def main():
    args = sys.argv[1:]
    if len(args) < 1:
        sys.exit(__doc__)
    command = args[0]
    outfiledir = 'DataSink'
    outfilestem = 'junk'
    if os.path.exists(outfiledir):
        if not os.path.isdir(outfiledir):
            sys.exit('\n"{}" exists and is not a directory.\n'.format(
                outfiledir))
    else:
        os.mkdir(outfiledir)
    if command == 'async':
        test_async(outfiledir, outfilestem)
    elif command == 'serial':
        test_serial(outfiledir, outfilestem)
    else:
        sys.exit(__doc__)


if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()
